
# q-expansion of newform 6762.2.a.cl, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6762
weight = 2
poly_data = [30, 8, -13, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-7, -2, 1, 0], 1], [[10, -8, -3, 1], 2]]

hecke_ring_character_values = None
aps_data = [[1, 0, 0, 0], [-1, 0, 0, 0], [1, -1, 0, 0], [0, 0, 0, 0], [0, 1, 0, 1], [2, 0, 1, 0], [1, 0, 0, 1], [-2, 0, 0, 0], [-1, 0, 0, 0], [3, 0, -1, 0], [-2, 0, 0, 0], [6, -2, 0, 0], [0, 2, 0, 0], [2, 0, 0, 0], [-1, -2, 0, -2], [5, 1, 2, 0], [4, -4, 0, 0], [0, 0, -2, 0], [-1, -1, -2, 0], [1, 0, 2, -2], [1, 0, -2, 0], [8, -1, 0, -1], [-2, 0, 0, -2], [-2, 2, 2, 0], [-8, 0, 0, 0], [7, 0, 1, 0], [-3, 4, 2, -1], [8, 2, 0, 2], [4, -1, -2, 1], [1, -1, 0, -2], [0, -2, -2, 0], [-6, 2, -1, -2], [3, 2, 0, -3], [3, 0, 1, 2], [7, -3, 0, -2], [-2, -2, 2, 0], [10, 1, 2, 1], [-1, 0, -1, 2], [0, 0, -3, 0], [-5, -2, -1, 4], [0, 2, -1, 4], [-6, -3, 2, -3], [-2, 2, 2, 0], [-2, 4, -1, 0], [13, 0, 3, 0], [-4, 3, 0, -1], [5, 2, -1, 2], [-12, 6, 0, 0], [8, 3, 0, -1], [4, 5, 2, 3], [-4, 4, 0, 4], [-9, 0, -1, 2], [10, 4, 0, 2], [4, 1, 4, -3], [14, 4, 0, 2], [6, 2, -2, 2], [-1, -2, -1, -4], [4, 2, 2, 2], [-8, -4, -1, -2], [3, -8, -2, -5], [5, 3, 0, -2], [9, -1, 0, 2], [-15, 2, 1, 2], [13, 2, 0, 2], [-4, 4, 0, 4], [-2, 8, 0, 0], [-6, 6, 4, 0], [10, 0, -2, 0], [-4, -2, -3, -4], [-16, 2, -1, 2], [24, 2, 0, 0], [10, 2, 0, -2], [3, -5, -2, -4], [2, 3, 2, 3], [5, -5, 2, 0], [6, 4, 0, -4], [-4, 0, 4, -2], [-10, 0, -3, -2], [9, -2, 0, -1], [-3, 2, -4, 0], [2, -3, -4, 3], [30, 1, 0, 1], [6, -4, 2, 4], [0, -6, -2, -6], [2, -6, -4, -2], [10, -10, -1, -2], [-8, 4, -2, -2], [-4, -4, 0, 0], [6, 0, 0, 0], [4, 0, 4, 0], [-4, 1, -2, -3], [8, 6, 0, 4], [-6, -4, 2, 2], [-8, 4, -1, 6], [-3, -2, 1, -6], [8, -6, -6, 2], [7, 4, 1, 4], [9, -9, -4, -2], [-7, 5, 0, 6], [4, -4, 1, 4], [7, -4, -3, 2], [-2, -8, -2, -6], [-18, 5, 0, -1], [-1, 1, 0, -6], [-9, -7, -2, 4], [-5, -2, -3, 0], [-4, -8, 3, -4], [-2, -4, -4, -2], [-21, -4, 2, -2], [-10, -4, 0, 0], [4, 6, -2, 0], [12, -1, 0, -3], [-5, 10, 4, 1], [-1, 1, 2, 6], [5, 8, 2, -1], [-15, 8, 2, 1], [-12, 0, 2, 0], [15, -4, -1, -2], [15, 2, -3, 4], [-6, -5, 2, -7], [-16, 9, 2, 3], [3, 10, 1, -4], [1, 5, 4, -2], [-10, 6, 7, -2], [9, 2, 3, -2], [-15, 1, 0, -2], [-8, -1, 2, 1], [-18, -6, -1, -4], [0, 9, 4, 1], [-20, -7, -2, -1], [-11, 0, -3, -6], [-4, 0, -2, 2], [-24, -2, -2, -6], [16, 0, 0, 0], [2, 2, 2, 2], [4, 10, 2, -2], [3, 1, -6, 2], [7, -7, 0, -4], [7, -5, -2, 6], [-12, 2, -6, 0], [9, 2, -1, 6], [13, 6, -1, 8], [18, 4, 4, 6], [-12, -7, 2, 1], [14, -8, 3, 0], [-8, 8, 0, -4], [0, -2, -2, 6], [16, -2, 2, 2], [-7, 0, 1, 2], [-5, -2, 0, 2], [-2, 2, -5, 0], [-1, 0, -2, -5], [8, -12, 4, 0], [-1, 12, -1, 6], [-23, 3, 0, 2], [-18, 2, 6, 0], [-3, -2, 2, 1], [4, 10, 0, 10], [20, -10, 0, -4], [-4, -6, -4, 4], [-6, 12, 3, 2], [8, -3, -2, -5], [-28, 18, 4, 0], [-4, -9, -2, -1], [-11, -1, 2, -2], [-36, -2, 2, 0], [22, -6, 0, -6], [4, -4, -2, -8], [20, 4, 6, -2], [-6, -4, -2, -4], [-8, -2, -6, 2], [-12, 12, 1, 6], [-18, -6, -6, 0], [12, 4, 4, 4], [-10, -4, 0, 0], [24, 5, -2, 3], [-43, -2, 1, -2], [7, 4, 1, 8], [-19, 5, 0, -4], [-28, 5, -2, -1], [8, -2, 0, -12], [4, 1, 2, -7], [6, 6, 1, 4], [-8, 14, 2, 0], [8, 10, 0, -6], [-7, -3, -4, -4], [-4, 8, -7, 2], [20, 2, -2, 6], [3, 8, 0, -4], [-26, 18, 5, 0], [5, 2, 7, 0], [16, -14, 0, -2], [-25, 9, -4, -4], [22, 0, -2, 0], [-6, 5, 8, -5], [25, -12, -4, 1], [4, -2, -4, -4], [2, 3, 6, -1], [11, 1, -6, 2], [8, 4, -3, -4], [-21, 15, -2, -2], [-20, -2, 0, 8], [2, -2, -7, 0], [-10, 10, 2, 8], [34, -1, 2, 1], [-28, 6, 2, 0], [5, 5, 6, -4], [-32, -6, 5, -4], [-16, -6, -2, -8], [-28, -4, 2, -4], [-16, -10, 5, -8], [-17, 12, -1, -4], [-12, 11, -4, 3], [-10, 4, -4, 10], [29, 4, 5, 6], [-36, 0, -8, -2], [10, 0, -6, 6], [0, 4, 2, -6], [16, -10, 4, 8], [-9, 8, 7, 0], [-18, -2, -3, -6], [17, 6, 4, -11], [-16, -2, -6, -2], [3, -6, -4, 3], [-26, 2, -2, 2], [-2, 8, -1, 4], [2, -6, -2, -8], [27, -2, -2, 2], [31, -8, -2, -7], [-2, -10, -1, 2], [10, -12, 3, 8], [-5, 5, 6, 4], [-8, 3, -4, 11], [-22, 8, 4, -2], [-45, 1, -2, -2], [-16, 14, 0, 0], [-38, -4, 2, 0], [-12, -8, 4, -10], [-14, 0, -1, -4], [20, -4, -4, -4], [16, -1, 4, 11], [-17, 6, -5, -2], [-46, 2, -4, -2], [4, 0, -11, 4], [-2, 2, -4, -8], [21, 0, 5, -2], [12, -18, -6, -8], [0, -6, -8, -2], [-12, 18, 2, -2], [-24, 6, -8, -6], [10, 14, -4, 12], [5, -2, -2, 3], [-8, 20, -4, 0], [-10, 14, 2, 10], [36, 2, -2, 0], [-12, -12, 0, -8], [0, 5, 4, -5], [8, -10, -6, -6], [-28, 2, 0, 8], [-17, 14, -3, 4], [-1, -10, -8, -9], [-22, -8, 0, 2], [-10, -4, -3, 2], [28, 1, 0, 3], [36, -4, 10, 6], [-34, -14, 4, -10], [35, -1, 0, -2], [12, -1, -6, 5], [-9, -3, 2, 0], [17, -18, -1, 0], [-34, 18, 7, 0], [10, -14, 4, -2], [-4, 0, 8, -8], [17, -11, 2, 4], [12, 10, 1, 4], [26, -8, 2, -2], [6, 2, 1, 8], [-20, -10, -1, -12], [16, -4, 0, 8], [18, -13, -2, -3], [10, 2, -3, -8], [19, -17, -6, -8], [-16, -10, 2, 4], [-54, 2, 2, -4], [37, -6, -1, -2], [39, -8, -1, -2], [42, 4, -2, 0], [7, -1, -12, 6], [-2, 12, 0, 6], [30, -4, -2, 2], [18, 4, 2, 2], [-22, -3, 0, 3], [22, 8, 6, 6], [24, -6, -2, -2], [0, -2, -5, 6], [-17, -7, 4, -6], [33, 0, 0, -5], [1, 10, 9, 8], [8, -6, 1, -2], [-5, -10, 1, -6], [10, 10, -6, 14], [-35, 19, 10, 2], [-15, -4, -8, 3], [10, 10, -2, -2], [43, 5, 0, 2], [7, -6, 5, 8], [6, 8, 7, 4], [16, -20, -4, 0], [28, -4, 2, 8], [24, -12, -2, -8], [15, 16, 3, 6], [-21, 1, -4, 0], [-9, -16, 4, -5], [13, 14, -3, 2], [-12, 4, -2, -4], [22, 10, -8, 2], [6, -2, -10, -2], [15, -4, 7, 2], [-6, -10, -10, 2], [50, -6, 4, 2], [1, -19, 2, -2], [-10, 4, 10, -10], [0, -10, -2, -12], [2, 6, -2, 4], [41, -10, -2, -9], [28, -8, -2, -4], [44, -8, 10, 8], [3, -8, 5, 10], [-22, 4, -10, -2], [28, 8, 4, 10], [2, 8, -3, -2], [-23, -8, 1, -4], [16, -24, -2, -6], [20, -6, 9, 6], [1, -1, 4, -8], [-22, 2, 5, -4], [14, -11, -4, -7], [4, 11, 6, 1], [-46, 0, 3, 0], [4, -3, 4, 1], [16, 4, 5, -12], [-5, 13, -6, 6], [7, -2, -11, 0], [-20, -6, -2, -6], [38, 0, -10, 6], [-18, 14, 5, -2], [-26, -12, -7, -16], [-43, -8, -1, -10], [-20, -4, -4, 6], [30, 2, 2, -4], [-41, -8, -7, -2], [39, 10, -2, 12], [13, 4, 1, -12], [-21, 4, -8, 1], [36, 3, 8, -11], [-8, -8, 6, -12], [2, 6, 12, -4], [4, 1, -8, 13], [-6, -12, -6, 2], [39, 6, 6, 6], [16, 17, -4, -1], [4, -20, -2, -10], [-9, -16, -1, -6], [14, 1, -6, 3], [34, 10, 6, 8], [-4, -10, -9, -4], [36, 6, 12, 12], [32, 10, 2, 6], [-14, -4, 4, -6], [22, -20, 2, 4], [8, -12, -8, -12], [15, 12, 12, 6], [-6, 8, 2, 10], [-35, -12, 6, -9], [30, -18, -1, -8], [24, -6, 0, 6], [28, 12, 4, 6], [24, 4, 6, -8], [-61, 18, 4, 1], [-38, -12, 4, 0], [-14, -6, 2, -4], [-21, -4, -10, -1], [-10, -12, -8, -10], [6, -16, -16, 2], [75, -7, 0, 2], [-12, -12, -1, 8], [-40, 0, -2, 10], [10, -10, 4, -2], [23, -9, -10, -2], [-54, 6, 0, -6], [-20, 2, -10, 6], [-35, 2, -9, 4], [-22, -12, -3, 14], [-39, -8, 3, -2], [46, 6, 8, -2], [36, -10, 0, 4], [15, -20, -2, -3], [9, -1, -10, -2], [-49, 3, 4, -2], [29, -2, 7, -8], [4, -14, 3, -4], [50, 8, 0, 2], [-16, -5, -4, 11], [-33, -7, -10, -14], [-3, -4, -11, 0], [-32, 4, 4, 6], [16, -3, 4, 7], [38, -10, -4, 6], [1, 13, 4, 14], [-42, 6, -6, 2], [-15, 22, 8, 0], [49, -2, 1, 0], [22, -3, 2, 11], [-5, -9, -6, -14], [24, 12, 6, 2], [-32, 10, -4, 8], [-49, -2, 0, -6], [-6, 24, 8, 16], [-57, 12, 6, 3], [-12, -2, -4, 6], [-34, 2, 6, -4], [6, -5, -10, 7], [-17, -20, 3, -2], [-18, -8, -15, 10], [32, 2, -9, 4], [7, 12, 11, 0], [-32, 2, -2, 10], [16, 16, -6, 0], [23, -4, -7, 6], [39, 0, -1, -2]]
